@@ -849,13 +849,17 @@ dee_serializable_model_insert_row_sorted (DeeModel           *self,
                                           gpointer            user_data)
 {
   DeeModelIter *iter;
+  gboolean was_found;
 
   g_return_val_if_fail (DEE_IS_SERIALIZABLE_MODEL (self), NULL);
   g_return_val_if_fail (row_members != NULL, NULL);
   g_return_val_if_fail (cmp_func != NULL, NULL);
 
   iter = dee_model_find_row_sorted (self, row_members, cmp_func,
-                                    user_data, NULL);
+                                    user_data, &was_found);
+  if (was_found)
+    iter = dee_model_next (self, iter);
+
   return dee_model_insert_row_before (self, iter, row_members);
 }
 
@@ -866,14 +870,16 @@ dee_serializable_model_find_row_sorted   (DeeModel           *self,
                                           gpointer            user_data,
                                           gboolean           *out_was_found)
 {
-  DeeModelIter  *iter, *end;
+  DeeModelIter  *iter, *end, *last_matching;
   GVariant     **row_buf;
+  gint           cmp_result;
   guint          n_cols, i;
 
   g_return_val_if_fail (DEE_IS_SERIALIZABLE_MODEL (self), NULL);
   g_return_val_if_fail (row_spec != NULL, NULL);
   g_return_val_if_fail (cmp_func != NULL, NULL);
 
+  last_matching = NULL;
   if (out_was_found != NULL) *out_was_found = FALSE;
   n_cols = dee_model_get_n_columns (self);
 
@@ -885,18 +891,31 @@ dee_serializable_model_find_row_sorted   (DeeModel           *self,
   while (iter != end)
     {
       dee_model_get_row (self, iter, row_buf);
-      if (cmp_func(row_buf, row_spec, user_data) >= 0)
+      cmp_result = cmp_func (row_buf, row_spec, user_data);
+      /* we're returning last matching row to make ordering
+       * of insert_row_sorted stable and fast */
+      while (cmp_result == 0)
         {
+          last_matching = iter;
+          iter = dee_model_next (self, iter);
+          if (iter == end)
+            {
+              iter = last_matching;
+              break;
+            }
           for (i = 0; i < n_cols; i++) g_variant_unref (row_buf[i]);
-          if (out_was_found != NULL) *out_was_found = TRUE;
-          break;
+          dee_model_get_row (self, iter, row_buf);
+          cmp_result = cmp_func (row_buf, row_spec, user_data);
         }
 
       for (i = 0; i < n_cols; i++) g_variant_unref (row_buf[i]);
+      /* if we're past the matching row, the loop can be quit */
+      if (cmp_result >= 0) break;
       iter = dee_model_next (self, iter);
     }
 
-  return iter;
+  if (out_was_found != NULL && last_matching != NULL) *out_was_found = TRUE;
+  return last_matching ? last_matching : iter;
 }
 
 static void
