@@ -106,6 +106,7 @@ struct _DeeSharedModelPrivate
   gboolean    clone_in_progress;
 
   DeeSharedModelAccessMode access_mode;
+  DeeSharedModelFlushMode flush_mode;
 };
 
 typedef struct
@@ -134,6 +135,7 @@ enum
   PROP_SYNCHRONIZED,
   PROP_DISABLE_REMOTE_WRITES,
   PROP_ACCESS_MODE,
+  PROP_FLUSH_MODE,
 };
 
 typedef enum
@@ -462,7 +464,8 @@ enqueue_revision (DeeModel  *self,
   priv->revision_queue = g_slist_prepend (priv->revision_queue, rev);
 
   /* Flush the revision queue once in idle */
-  if (priv->revision_queue_timeout_id == 0)
+  if (priv->revision_queue_timeout_id == 0 &&
+      priv->flush_mode == DEE_SHARED_MODEL_FLUSH_MODE_AUTOMATIC)
     {
       priv->revision_queue_timeout_id =
         g_idle_add ((GSourceFunc)flush_revision_queue_timeout_cb, self);
@@ -563,6 +566,15 @@ dee_shared_model_set_property (GObject      *object,
     case PROP_ACCESS_MODE:
       priv->access_mode = g_value_get_enum (value);
       break;
+    case PROP_FLUSH_MODE:
+      priv->flush_mode = g_value_get_enum (value);
+      if (priv->flush_mode != DEE_SHARED_MODEL_FLUSH_MODE_AUTOMATIC &&
+          priv->revision_queue_timeout_id != 0)
+        {
+          g_source_remove (priv->revision_queue_timeout_id);
+          priv->revision_queue_timeout_id = 0;
+        }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, id, pspec);
       break;
@@ -590,6 +602,9 @@ dee_shared_model_get_property (GObject    *object,
     case PROP_ACCESS_MODE:
       g_value_set_enum (value, priv->access_mode);
       break;
+    case PROP_FLUSH_MODE:
+      g_value_set_enum (value, priv->flush_mode);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, id, pspec);
       break;
@@ -724,6 +739,25 @@ dee_shared_model_class_init (DeeSharedModelClass *klass)
                              | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (obj_class, PROP_ACCESS_MODE, pspec);
 
+  /**
+   * DeeSharedModel:flush-mode:
+   *
+   * Enumeration defining the flushing behavior.
+   *
+   * Setting this to #DEE_SHARED_MODEL_FLUSH_MODE_MANUAL will disable 
+   * automatic flushing that usually happens when the application's main event
+   * loop is idle. Automatic flushing should be primarily disabled when 
+   * a shared model is used from multiple threads, or when not using #GMainLoop.
+   * When disabled, dee_shared_model_flush_revision_queue() needs to be called
+   * explicitely.
+   */
+  pspec = g_param_spec_enum ("flush-mode", "Flush mode",
+                             "Determines whether flushes occur automatically",
+                             DEE_TYPE_SHARED_MODEL_FLUSH_MODE,
+                             DEE_SHARED_MODEL_FLUSH_MODE_AUTOMATIC,
+                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (obj_class, PROP_FLUSH_MODE, pspec);
+
   /**
    * DeeSharedModel::begin-transaction:
    * @model: The shared model the signal is emitted on
@@ -832,8 +866,10 @@ handle_dbus_method_call (GDBusConnection       *connection,
       else
         {
           // FIXME: It can be expensive to build the clone. Perhaps thread this?
-          retval = dee_serializable_serialize(DEE_SERIALIZABLE (user_data));
+          retval = dee_serializable_serialize (DEE_SERIALIZABLE (user_data));
           g_dbus_method_invocation_return_value (invocation, retval);
+          /* dee_serializable_serialize returns full ref, unref it */
+          g_variant_unref (retval);
         }
     }
   else if (g_strcmp0 ("Invalidate", method_name) == 0)
@@ -1656,6 +1692,34 @@ GType dee_shared_model_access_mode_get_type (void)
   return shared_model_access_mode_type;
 }
 
+GType dee_shared_model_flush_mode_get_type (void)
+{
+  static GType shared_model_flush_mode_type = 0;
+  if (shared_model_flush_mode_type == 0)
+    {
+      static const GEnumValue values[] =
+      {
+        {
+          DEE_SHARED_MODEL_FLUSH_MODE_AUTOMATIC,
+          "DEE_SHARED_MODEL_FLUSH_MODE_AUTOMATIC",
+          "automatic"
+        },
+        {
+          DEE_SHARED_MODEL_FLUSH_MODE_MANUAL,
+          "DEE_SHARED_MODEL_FLUSH_MODE_MANUAL",
+          "manual"
+        },
+        {
+          0, NULL, NULL
+        }
+      };
+      shared_model_flush_mode_type =
+        g_enum_register_static ("DeeSharedModelFlushMode", values);
+    }
+
+  return shared_model_flush_mode_type;
+}
+
 /**
  * dee_shared_model_new:
  * @name: A well known name to publish this model under. Models sharing this name
@@ -1802,6 +1866,39 @@ dee_shared_model_get_peer (DeeSharedModel *self)
   return priv->swarm;
 }
 
+/**
+ * dee_shared_model_get_flush_mode:
+ * @self: A #DeeSharedModel
+ *
+ * Convenience function for accessing the #DeeSharedModel:flush-mode property.
+ *
+ * Returns: (transfer none): The #DeeSharedModelFlushMode used by the model
+ */
+DeeSharedModelFlushMode
+dee_shared_model_get_flush_mode (DeeSharedModel *self)
+{
+  g_return_val_if_fail (DEE_IS_SHARED_MODEL (self),
+                        DEE_SHARED_MODEL_FLUSH_MODE_MANUAL);
+
+  return self->priv->flush_mode;
+}
+
+/**
+ * dee_shared_model_set_flush_mode:
+ * @self: A #DeeSharedModel
+ * @mode: Flush mode to use
+ *
+ * Convenience function for setting the #DeeSharedModel:flush-mode property.
+ */
+void
+dee_shared_model_set_flush_mode (DeeSharedModel *self,
+                                 DeeSharedModelFlushMode mode)
+{
+  g_return_if_fail (DEE_IS_SHARED_MODEL (self));
+
+  g_object_set (self, "flush-mode", mode, NULL);
+}
+
 /**
  * dee_shared_model_is_leader:
  * @self: The model to inspect
