@@ -62,6 +62,7 @@ static void test_remote_append  (Fixture *fix, gconstpointer data);
 static void test_disabled_writes (Fixture *fix, gconstpointer data);
 static void test_commit_before_clone (Fixture *fix, gconstpointer data);
 static void test_force_resync   (Fixture *fix, gconstpointer data);
+static void test_manual_flush   (Fixture *fix, gconstpointer data);
 
 void
 test_model_interactions_create_suite (void)
@@ -106,6 +107,8 @@ test_model_interactions_create_suite (void)
               model_setup, test_commit_before_clone, model_teardown);
   g_test_add (DOMAIN"/ForceResync", Fixture, 0,
               model_setup, test_force_resync, model_teardown);
+  g_test_add (DOMAIN"/ManualFlush", Fixture, 0,
+              model_setup, test_manual_flush, model_teardown);
 }
 
 static void
@@ -898,3 +901,18 @@ test_force_resync (Fixture *fix, gconstpointer data)
   g_assert_cmpint (dee_model_get_n_rows (fix->model), ==, 3);
 }
 
+static void
+test_manual_flush (Fixture *fix, gconstpointer data)
+{
+  DeeSharedModel *sm;
+  sm = DEE_SHARED_MODEL (fix->model);
+  dee_shared_model_set_flush_mode (sm, DEE_SHARED_MODEL_FLUSH_MODE_MANUAL);
+  _add5rows (fix->model);
+
+  /* A timeout would normally cause the model to flush, but with manual
+   * flushing it shouldn't */
+  gtx_yield_main_loop (50); /* 50ms yield */
+
+  g_assert_cmpuint (dee_shared_model_flush_revision_queue (sm), >, 0);
+}
+
